declare enum RegistrationType {
    Class = "class",
    Factory = "factory",
    Instance = "instance"
}

declare enum ScopeType {
    Transient = "transient",
    Request = "request",
    Singleton = "singleton"
}

/**
 * Creates, wires dependencies and manages lifetime for a set of services.
 * Most instances of Container are created by a [[ContainerBuilder]].
 */
declare class Container {
    private readonly services;
    private readonly singletons;
    /**
     * @internal
     */
    constructor(services: ReadonlyMap<Identifier<unknown>, ServiceData<unknown>>);
    /**
     * Gets the service object of the registered identifier.
     * @param identifier Class of the service to get.
     * @typeParam T The type of the service.
     * @returns
     */
    get<T>(identifier: Identifier<T>): T;
    private getService;
    private findServiceDataOrThrow;
    private getDependencies;
}

/**
 * Represents a newable class.
 * @typeParam T Class type.
 */
interface Newable<T> extends Function {
    new (...args: any[]): T;
}
/**
 * Represents an abstract class.
 * @typeParam T Class type.
 */
interface Abstract<T> extends Function {
    prototype: T;
}
/**
 * Service identifier. Can be a concrete implementation or an abstraction.
 * @typeParam T Class type.
 */
declare type Identifier<T> = Newable<T> | Abstract<T>;
/**
 * Represents a factory to create instances of a class.
 * The [[Container]] will be passed to the registered factory to be able to
 * get any other service.
 * @typeParam T Class type.
 */
declare type Factory<T> = (container: Container) => T;
/**
 * Service instance.
 * @internal
 */
declare type Instance<T extends Object> = T;
/**
 * Options for the [[Container]] build method.
 */
declare type BuildOptions = {
    /** Whether to autowire dependencies based on types or not. Default value: `true`.  */
    autowire?: boolean;
};

declare type ClassServiceData<T> = {
    scope: ScopeType;
    class: Newable<T>;
    autowire: boolean;
    type: RegistrationType.Class;
    dependencies: Array<Abstract<unknown>>;
};
declare type FactoryServiceData<T> = {
    scope: ScopeType;
    factory: Factory<T>;
    dependencies: never[];
    type: RegistrationType.Factory;
};
declare type InstanceServiceData<T> = {
    scope: ScopeType.Singleton;
    instance: Instance<T>;
    type: RegistrationType.Instance;
    dependencies: never[];
};
declare type ServiceData<T> = ClassServiceData<T> | FactoryServiceData<T> | InstanceServiceData<T>;
declare type Buildable<C, T> = {
    instance: C;
    build: (options: BuildOptions) => ServiceData<T>;
};

declare abstract class ServiceConfiguration<T> {
    protected abstract scope: ScopeType;
    protected abstract build(options: BuildOptions): ServiceData<T>;
    protected asTransient(): this;
    protected asSingleton(): this;
    protected asInstancePerRequest(): this;
}

interface WithScopeChange<T> {
    asTransient(): T;
    asSingleton(): T;
    asInstancePerRequest(): T;
}

declare class ClassConfiguration<T> extends ServiceConfiguration<T> implements WithScopeChange<ClassConfiguration<T>> {
    private readonly newable;
    protected scope: ScopeType;
    private dependencies;
    private autowire;
    private constructor();
    /**
     * Declare class dependencies manually
     * @param dependencies List of class dependency identifiers to inject in
     * order to the constructor.
     * @returns
     */
    withDependencies(dependencies: Array<Identifier<unknown>>): this;
    /**
     * Configure the service so that always gets a new instance.
     * @returns
     */
    asTransient(): this;
    /**
     * Configure the service so that always gets the same, shared instance.
     * @returns
     */
    asSingleton(): this;
    /**
     * Configure the service so that the same shared instance is used during
     * within a [[Container]].get request.
     * @returns
     */
    asInstancePerRequest(): this;
    private setDependencyInformationIfNotExist;
    protected build(options: BuildOptions): ServiceData<T>;
    /**
     * @internal
     */
    static createBuildable<TIdentifier>(newable: Newable<TIdentifier>): Buildable<ClassConfiguration<TIdentifier>, TIdentifier>;
}

declare class FactoryConfiguration<T> extends ServiceConfiguration<T> implements WithScopeChange<FactoryConfiguration<T>> {
    private readonly factory;
    protected scope: ScopeType;
    private constructor();
    /**
     * Configure the service so that always gets a new instance.
     * @returns
     */
    asTransient(): this;
    /**
     * Configure the service so that always gets the same, shared instance.
     * @returns
     */
    asSingleton(): this;
    /**
     * Configure the service so that the same shared instance is used during
     * within a [[Container]].get request.
     * @returns
     */
    asInstancePerRequest(): this;
    protected build(): ServiceData<T>;
    /**
     * @internal
     */
    static createBuildable<TIdentifier>(factory: Factory<TIdentifier>): Buildable<FactoryConfiguration<TIdentifier>, TIdentifier>;
}

declare class Registration<T> {
    readonly identifier: Identifier<T>;
    private buildable;
    private constructor();
    /**
     * Configure the class implementation that the identifier will provide.
     * @param newable The implementation that the identifier will provide.
     * @returns
     */
    useClass(newable: Newable<T>): ClassConfiguration<T>;
    /**
     * Configure the class implementation that the identifier will provide.
     * Alias of `useClass`.
     * @param newable The implementation that the identifier will provide.
     * @returns Configuration fluent API for classes
     */
    use(newable: Newable<T>): ClassConfiguration<T>;
    /**
     * Configure the instance that the identifier will provide.
     * @param instance The instance that the identifier will provide.
     */
    useInstance(instance: Instance<T>): void;
    /**
     * Configure a factory that returns the instance that the identifier will provide.
     * @param factory The factory that will be executed when the identifier is requested.
     * @returns Configuration fluent API for factories
     */
    useFactory(factory: Factory<T>): FactoryConfiguration<T>;
    private build;
    /**
     * @internal
     */
    static createBuildable<TIdentifier>(identifier: Identifier<TIdentifier>): Buildable<Registration<TIdentifier>, TIdentifier>;
}

/**
 * Used to build an [[Container]] from service registrations.
 */
declare class ContainerBuilder {
    private readonly buildables;
    /**
     * Registers a service.
     * @param identifier The class that identifies this service. This class
     * identifier must be used to get the service from the container or when
     * defining it as a dependency.
     * @typeParam T The type of the service.
     * @returns
     */
    register<T>(identifier: Identifier<T>): Registration<T>;
    /**
     * Unregister previously registered service.
     * @param identifier The class that identifies this service to be unregistered.
     * @typeParam T The type of the service.
     */
    unregister<T>(identifier: Identifier<T>): void;
    /**
     * Checks whether a service is registered or not.
     * @param identifier The class that identifies this service to be checked.
     * @typeParam T The type of the service.
     * @returns
     */
    isRegistered<T>(identifier: Identifier<T>): boolean;
    /**
     * Alias for `.register(newable).use(newable)`.
     * @param newable The concrete class implementation to be registered as itself.
     * @typeParam T The type of the service.
     * @returns
     */
    registerAndUse<T>(newable: Newable<T>): ClassConfiguration<T>;
    /**
     * Builds an inmutable [[Container]].
     * @param options Build options.
     * @returns
     */
    build({ autowire }?: BuildOptions): Container;
}

/**
 * Decorator for injectable classes. Every registered service must
 * be decorated because without decorators Typescript won't emit
 * constructor metadata.
 * @returns
 */
declare const Service: () => ClassDecorator;

export { Abstract, BuildOptions, ClassConfiguration, Container, ContainerBuilder, Factory, FactoryConfiguration, Identifier, Instance, Newable, Registration, Service };
